using Microsoft.Extensions.Configuration;
using System.Text.Json;
using System.Text.Json.Serialization;

namespace View_by_Distance.Rename.Models.Binder;

public class AppSettings
{

    public string? Company { get; set; }
    public int? MaxDegreeOfParallelism { get; set; }
    public bool? RequireRootDirectoryExists { get; set; }

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, BinderAppSettingsSourceGenerationContext.Default.AppSettings);
        return result;
    }

    private static void PreVerify(IConfigurationRoot configurationRoot, AppSettings? appSettings)
    {
        if (appSettings?.Company is null)
        {
            foreach (IConfigurationProvider configurationProvider in configurationRoot.Providers)
            {
                if (configurationProvider is not Microsoft.Extensions.Configuration.Json.JsonConfigurationProvider jsonConfigurationProvider)
                    continue;
                if (jsonConfigurationProvider.Source.FileProvider is not Microsoft.Extensions.FileProviders.PhysicalFileProvider physicalFileProvider)
                    continue;
                if (!physicalFileProvider.Root.Contains("UserSecrets"))
                    continue;
                throw new NotSupportedException(physicalFileProvider.Root);
            }
            throw new NotSupportedException("Not Found!");
        }
    }

    private static void Verify(AppSettings _)
    {
    }

    private static Models.AppSettings Get(AppSettings? appSettings)
    {
        Models.AppSettings result;
        if (appSettings is null) throw new NullReferenceException(nameof(appSettings));
        if (appSettings.Company is null) throw new NullReferenceException(nameof(appSettings.Company));
        if (appSettings.MaxDegreeOfParallelism is null) throw new NullReferenceException(nameof(appSettings.MaxDegreeOfParallelism));
        if (appSettings.RequireRootDirectoryExists is null) throw new NullReferenceException(nameof(appSettings.RequireRootDirectoryExists));
        Verify(appSettings);
        result = new(appSettings.Company,
                     appSettings.MaxDegreeOfParallelism.Value,
                     appSettings.RequireRootDirectoryExists.Value);
        return result;
    }

    public static Models.AppSettings Get(IConfigurationRoot configurationRoot)
    {
        Models.AppSettings result;
#pragma warning disable IL3050, IL2026
        AppSettings? appSettings = configurationRoot.Get<AppSettings>();
#pragma warning restore IL3050, IL2026
        PreVerify(configurationRoot, appSettings);
        result = Get(appSettings);
        return result;
    }

}

[JsonSourceGenerationOptions(WriteIndented = true)]
[JsonSerializable(typeof(AppSettings))]
internal partial class BinderAppSettingsSourceGenerationContext : JsonSerializerContext
{
}